/*
  prime_util.h

  Copyright (C) 2015 Taiji Yamada <taiji@aihara.co.jp>
*/
#ifndef _PRIME_UTIL_H_
#define _PRIME_UTIL_H_
#include <stdlib.h>
#include <string.h>	/* memset, etc. */
#include <math.h>	/* sqrt, etc. */
#include <fcntl.h>	/* open, etc. */
#include <unistd.h>	/* read, write, etc. */
#include <sys/mman.h>	/* mmap, etc */
#include <sys/stat.h>	/* fstat, etc */

#ifndef min
#define min(a, b) (((a) < (b)) ? (a) : (b))
#endif
#ifndef max
#define max(a, b) (((a) > (b)) ? (a) : (b))
#endif

#if !defined(USE_128_BIT_TYPE)
typedef unsigned long long int huge_uint;
typedef long long int huge_int;
#else
typedef __uint128_t huge_uint;
typedef __int128_t huge_int;
#endif

inline
huge_uint greatest_common_divisor(huge_uint m, huge_uint n)
{
  while (m) {
    huge_uint l = m;
    m = n % m;
    n = l;
  }
  return n;
}
inline
huge_uint least_common_multiple(huge_uint m, huge_uint n)
{
  return m/greatest_common_divisor(m, n)*n;
}
inline
huge_uint power(const huge_uint n, const huge_uint m)
{
  huge_uint p = 1;
  for (size_t i=0; i<m; ++i)
    p *= n;
  return p;
}
static inline
huge_uint binomial_coefficient(huge_uint n, huge_uint k)
{
  huge_uint c = 1, i;

  if (k > n) return 0;
  k = min(k, n - k);
  for (i=0; i<k; ++i)
    c = c*(n - i)/(i + 1);
  return c;
}

inline
huge_uint function_hilbert(const huge_uint i)
{
  return 4*i + 1;
}
inline
huge_uint function_pronic(const huge_uint i)
{
  return i*(i+1);
}
inline
huge_uint function_power_of_two(const huge_uint i)
{
  huge_uint n;

  n = power(2, i);
  return n;
}
inline
huge_uint function_cullen(const huge_uint i)
{
  return function_power_of_two(i)*i + 1;
}
inline
huge_uint function_woodall(const huge_uint i)
{
  return function_power_of_two(i)*i - 1;
}
inline
huge_uint function_triangular(const huge_uint i)
{
  return (i * (i+1))/2;
}
inline
huge_uint function_tetrahedral(const huge_uint i)
{
  return (i * (i+1) * (i+2))/6;
}
inline
huge_uint function_pentatope(const huge_uint i)
{
  return (i * (i+1) * (i+2) * (i+3))/24;
}
inline
huge_uint function_pentagonal(const huge_uint i)
{
  return (i * (i*3-1))/2;
}
inline
_Bool is_square_triangular(const huge_uint n)
{
  huge_uint r1, r2;

  r1 = sqrt(n);
  r2 = n - r1*r1;
  return r2 == 0;
}
inline
huge_uint function_squared_triangular(const huge_uint i)
{
  return ((i*i + i)/2)*((i*i + i)/2);
}
inline
huge_uint function_lazy_caterer(const huge_uint i)
{
  return (i*i + i + 2)/2;
}
inline
huge_uint function_jacobsthal(const huge_uint i)
{
  return (function_power_of_two(i) - ((i%2)?-1:1))/3;
}
inline
huge_uint function_jacobsthal_lucas(const huge_uint i)
{
  return function_power_of_two(i) + ((i%2)?-1:1);
}
inline
huge_uint function_kynea(const huge_uint i)
{
  huge_uint n = function_power_of_two(i) + 1;

  return n*n - 2;
}
inline
huge_uint function_thabit(const huge_uint i)
{
  return 3*function_power_of_two(i) - 1;
}
inline
huge_uint function_fermat(const huge_uint i)
{
  return function_power_of_two(function_power_of_two(i)) + 1; // ! too huge
}
inline
huge_uint function_catalan(const huge_uint i)
{
  return binomial_coefficient(2*i, i) / (i+1);
}

static inline
_Bool is_square_free(const huge_uint n)
{
  huge_uint i, sqrt_n;

  sqrt_n = sqrt(n);
  for (i=2; i<=sqrt_n; ++i)
    if (n % (i*i) == 0)
      break;
  return i == sqrt_n+1;
}
static inline
_Bool is_cube_free(const huge_uint n)
{
  huge_uint i, cbrt_n;

  cbrt_n = cbrt(n);
  for (i=2; i<=cbrt_n; ++i)
    if (n % (i*i*i) == 0)
      break;
  return i == cbrt_n+1;
}

static inline
huge_uint *divisors(const huge_uint n, huge_uint *n_return)
{
  huge_uint d = 1, sqrt_n;
  huge_uint *q = NULL;
  size_t n_q = 0;

  sqrt_n = sqrt(n);
  for (; d<=sqrt_n; ++d)
    if (n%d == 0) {
      if ((q = q ? realloc(q, sizeof(huge_uint)*(n_q+1)) : malloc(sizeof(huge_uint)*(n_q+1)))) q[n_q++] = d;
      if (d != n/d) if ((q = q ? realloc(q, sizeof(huge_uint)*(n_q+1)) : malloc(sizeof(huge_uint)*(n_q+1)))) q[n_q++] = n/d;
    }
  *n_return = n_q;
  return q;
}
inline
huge_uint n_divisors(const huge_uint n)
{
  huge_uint d = 1, sqrt_n, c = 0;

  sqrt_n = sqrt(n);
  for (; d<=sqrt_n; ++d)
    if (n%d == 0) {
      ++c;
      if (d != n/d) ++c;
    }
  return c;
}
inline
huge_uint sum_divisors(const huge_uint n)
{
  huge_uint d = 1, sqrt_n, s = 0;

  sqrt_n = sqrt(n);
  for (; d<=sqrt_n; ++d)
    if (n%d == 0) {
      s += d;
      if (d != n/d) s += n/d;
    }
  return s;
}

inline
_Bool is_abundant(const huge_uint n)
{
  return sum_divisors(n) > n*2;
}
inline
_Bool is_perfect(const huge_uint n)
{
  return sum_divisors(n) - n == n;
}
inline
_Bool is_multiperfect(const huge_uint n)
{
  return sum_divisors(n) % n == 0;
}
inline
_Bool is_deficient(const huge_uint n)
{
  return sum_divisors(n) < n*2;
}
inline
_Bool is_superperfect(const huge_uint n)
{
  return sum_divisors(sum_divisors(n)) == n*2;
}
inline
_Bool is_hemiperfect(const huge_uint n, huge_uint *k)
{
  huge_uint s;
  return (s=sum_divisors(n))*2 % n == 0 && (*k = s*2/n) % 2 == 1;
}
inline
_Bool is_refactorable(const huge_uint n)
{
  return n % n_divisors(n) == 0;
}
inline
_Bool is_arithmetic(const huge_uint n, huge_uint *s, huge_uint *c)
{
  return (*s=sum_divisors(n)) % (*c=n_divisors(n)) == 0;
}

static inline
_Bool is_harmonic_divisor(const huge_uint n)
{
  huge_uint n_q, *q = divisors(n, &n_q);
  huge_uint d = 0, c = q[0];
  size_t i;

  for (i=1; i<n_q; ++i)
    c = least_common_multiple(c, q[i]);
  for (i=0; i<n_q; ++i)
    d += c/q[i];
  free(q);
  return (n_q*c) % d == 0;
}

static inline
huge_uint *proper_divisors(const huge_uint n, huge_uint *n_return)
{
  huge_uint d = 1, sqrt_n;
  huge_uint *q = NULL;
  size_t n_q = 0;

  sqrt_n = sqrt(n);
  for (; d<=sqrt_n; ++d)
    if (n%d == 0) {
      if (n != d) if ((q = q ? realloc(q, sizeof(huge_uint)*(n_q+1)) : malloc(sizeof(huge_uint)*(n_q+1)))) q[n_q++] = d;
      if (n != n/d && d != n/d) if ((q = q ? realloc(q, sizeof(huge_uint)*(n_q+1)) : malloc(sizeof(huge_uint)*(n_q+1)))) q[n_q++] = n/d;
    }
  *n_return = n_q;
  return q;
}
inline
huge_uint n_proper_divisors(const huge_uint n)
{
  huge_uint d = 1, sqrt_n, c = 0;

  sqrt_n = sqrt(n);
  for (; d<=sqrt_n; ++d)
    if (n%d == 0) {
      if (n != d) ++c;
      if (n != n/d && d != n/d) ++c;
    }
  return c;
}
inline
huge_uint sum_proper_divisors(const huge_uint n)
{
  huge_uint d = 1, sqrt_n, s = 0;

  sqrt_n = sqrt(n);
  for (; d<=sqrt_n; ++d)
    if (n%d == 0) {
      if (n != d) s += d;
      if (n != n/d && d != n/d) s += n/d;
    }
  return s;
}

static inline
_Bool is_primitive_abundant(const huge_uint n)
{
  if (!is_abundant(n)) return 0;
  huge_uint n_d, *d = proper_divisors(n, &n_d);
  size_t i;

  for (i=0; i<n_d; ++i)
    if (!(sum_divisors(d[i]) < d[i]*2))
      break;
  free(d);
  return i == n_d;
}
static inline
_Bool is_primitive_abundant_have_no_abundant_proper_divisor(const huge_uint n)
{
  if (!is_abundant(n)) return 0;
  huge_uint n_d, *d = proper_divisors(n, &n_d);
  size_t i;

  for (i=0; i<n_d; ++i)
    if ((sum_divisors(d[i]) > d[i]*2))
      break;
  free(d);
  return i == n_d;
}
static inline
_Bool is_pseudoperfect(const huge_uint n)
{
  huge_uint n_d, *d = proper_divisors(n, &n_d);
  huge_uint i, c;
  size_t j;

  c = 1U<<n_d;
  for (i=1; i<c; ++i) {
    huge_uint s = 0;
    for (j=0; j<n_d; ++j)
      if (i & 1U<<j)
	s += d[j];
    if (s == n) {
      free(d);
      return !0;
    }
  }
  free(d);
  return 0;
}
static inline
_Bool is_pseudoperfect_with_proper_divisors(const huge_uint n, huge_uint **d_return, huge_uint *n_d_return)
{
  huge_uint i, c;
  huge_uint n_d, *d;
  size_t j;

  d = proper_divisors(n, &n_d);
  *d_return = d;
  *n_d_return = n_d;
  c = 1U<<n_d;
  for (i=1; i<c; ++i) {
    huge_uint s = 0;
    for (j=0; j<n_d; ++j)
      if (i & 1U<<j)
	s += d[j];
    if (s == n)
      return !0;
  }
  return 0;
}
static inline
_Bool is_weird(const huge_uint n)
{
  if (!(sum_divisors(n) > n*2)) return 0;
  huge_uint n_d, *d = proper_divisors(n, &n_d);
  huge_uint i, c;

  c = 1U<<n_d;
  for (i=1; i<c; ++i) {
    huge_uint s = 0;
    for (size_t j=0; j<n_d; ++j)
      if (i & 1U<<j)
	s += d[j];
    if (s == n) break;
  }
  free(d);
  return i == c;
}
static inline
_Bool is_practical(const huge_uint n)
{
  huge_uint n_d, *d = proper_divisors(n, &n_d);
  huge_uint m, i, c;

  c = 1U<<n_d;
  for (m = 1; m < n; ++m) {
    for (i=1; i<c; ++i) {
      huge_uint s = 0;
      for (size_t j=0; j<n_d; ++j)
	if (i & 1U<<j)
	  s += d[j];
      if (s == m) break;
    }
    if (i == c) break;
  }
  free(d);
  return !(m < n);
}

inline
huge_uint *unitary_divisors(const huge_uint n, huge_uint *n_return)
{
  huge_uint d = 1, sqrt_n;
  huge_uint *q = NULL;
  size_t n_q = 0;

  sqrt_n = sqrt(n);
  for (; d<=sqrt_n; ++d)
    if (n%d == 0) {
      if (greatest_common_divisor(n/d, d) == 1) {
	if ((q = q ? realloc(q, sizeof(huge_uint)*(n_q+1)) : malloc(sizeof(huge_uint)*(n_q+1)))) q[n_q++] = d;
	if (d != 1) if ((q = q ? realloc(q, sizeof(huge_uint)*(n_q+1)) : malloc(sizeof(huge_uint)*(n_q+1)))) q[n_q++] = n/d;
      }
    }
  *n_return = n_q;
  return q;
}
inline
huge_uint n_unitary_divisors(const huge_uint n)
{
  huge_uint d = 1, sqrt_n, c = 0;

  sqrt_n = sqrt(n);
  for (; d<=sqrt_n; ++d)
    if (n%d == 0) {
      if (greatest_common_divisor(n/d, d) == 1) {
	++c;
	if (d != 1) ++c;
      }
    }
  return c;
}
inline
huge_uint sum_unitary_divisors(const huge_uint n)
{
  huge_uint d = 1, sqrt_n, s = 0;

  sqrt_n = sqrt(n);
  for (; d<=sqrt_n; ++d)
    if (n%d == 0) {
      if (greatest_common_divisor(n/d, d) == 1) {
	s += d;
	if (d != 1) s += n/d;
      }
    }
  return s;
}

inline
_Bool is_unitary_perfect(const huge_uint n)
{
  return n != 1 && sum_unitary_divisors(n) == n;
}

inline
huge_uint factorial(huge_uint n)
{
  huge_uint f = 1;

  if (n == 0) return 1;
  for (; n != 1; --n)
    f *= n;
  return f;
}
inline
huge_uint subfactorial(const huge_uint n)
{
  huge_uint f = 1, i;

  for (i=1; i<=n; ++i)
    f = i*f + ((i%2)?-1:1);
  return f;
}
inline
huge_uint semifactorial(const huge_uint n)
{
  huge_uint f = 1, i = (n % 2 == 1) ? 1 : 2;

  for (; i<=n; i+=2)
    f *= i;
  return f;
}

inline
_Bool isprime_simple(huge_uint n)
{
  if (n < 2) return 0;
  if (n == 2) return !0;
  if (!(n % 2)) return 0;
  const huge_uint sqrt_n = sqrt(n);
  huge_uint i;
  for (i = 3; i <= sqrt_n; i += 2)
    if (n % i == 0)
      return 0;
  return !0;
}

/*
  set prime numbers on array
*/
typedef _Bool isprime_table_element_t;
#define isprime_table_size(n) (sizeof(isprime_table_element_t)*(n))

/*
  set prime numbers by Eratosthenes's Sieve
*/
inline
void primes_eratosthenes_sieve(isprime_table_element_t *isprime_table, const huge_uint n)
{
  huge_uint i, j;
  const double sqrt_n = sqrt(n);

  isprime_table[0] = isprime_table[1] = 0;
  for (i = 2; i < n; i++) isprime_table[i] = !0;
  for (i = 2; i < sqrt_n; i++)
    for (j = 2; i * j < n; j++)
      isprime_table[i*j] = 0;
}
inline
huge_uint *prime_array(isprime_table_element_t *isprime_table, const huge_uint n, size_t *n_primes)
{
  huge_uint i, *a;
  size_t j = 0, c = 0;

  for (i=0; i<n; i++)
    if (isprime_table[i])
      c++;
  *n_primes = c;
  if (!(a = malloc(sizeof(huge_uint)*c)))
    return NULL;
  for (i=0; i<n; i++)
    if (isprime_table[i])
      a[j++] = i;
  return a;
}
inline
huge_uint *prime_array_with_indice(size_t *prime_array_indice, isprime_table_element_t *isprime_table, const huge_uint n, size_t *n_primes)
{
  huge_uint i, *a;
  size_t j = 0, c = 0;

  for (i=0; i<n; i++)
    if (isprime_table[i])
      c++;
  *n_primes = c;
  if (!(a = malloc(sizeof(huge_uint)*c)))
    return NULL;
  for (i=0; i<n; i++)
    if (isprime_table[i]) {
      prime_array_indice[i] = j;
      a[j++] = i;
    }
  return a;
}

/*
  set prime numbers on bits
*/
#if !defined(USE_128_BIT_TYPE)
typedef unsigned long long int isprime_bit_table_element_t;
#else
typedef __uint128_t isprime_bit_table_element_t;
#endif

#define isprime_bit_length (sizeof(isprime_bit_table_element_t)*8)

#if defined(DONT_USE_EVEN_OMISSION_IN_BIT_TABLE)
#define isprime_bit_table_length(n) ((n)/isprime_bit_length + ((n)%isprime_bit_length ? 1 : 0))
#define isprime_bit_table_upper(n) (isprime_bit_length*isprime_bit_table_length(n))
#define isprime_bit_table_number(z) ((z)*8)
#define isprime_bit_table_size(n) (sizeof(isprime_bit_table_element_t)*isprime_bit_table_length(n))
#define set_isprime_bit_table(n) (isprime_bit_table[(n)/isprime_bit_length] |= (1ULL<<((n)%isprime_bit_length)))
#define unset_isprime_bit_table(n) (isprime_bit_table[(n)/isprime_bit_length] &= ~(1ULL<<((n)%isprime_bit_length)))
#define isprime_bit(n) ((isprime_bit_table[(n)/isprime_bit_length] & (1ULL<<((n)%isprime_bit_length))) != 0ULL)
#else
#define isprime_bit_table_length(n) (((n)>>1)/isprime_bit_length + (((n)>>1)%isprime_bit_length ? 1 : 0))
#define isprime_bit_table_upper(n) (isprime_bit_length*isprime_bit_table_length(n)*2)
#define isprime_bit_table_number(z) ((z)*16)
#define isprime_bit_table_size(n) (sizeof(isprime_bit_table_element_t)*isprime_bit_table_length(n))
#define set_isprime_bit_table(n) do { if ((n)&1ULL) { huge_uint m = (n)>>1; isprime_bit_table[(m)/isprime_bit_length] |= (1ULL<<((m)%isprime_bit_length)); } } while (0)
#define unset_isprime_bit_table(n) do { if ((n)&1ULL) { huge_uint m = (n)>>1; isprime_bit_table[(m)/isprime_bit_length] &= ~(1ULL<<((m)%isprime_bit_length)); } } while (0)
#define isprime_bit(n) (((n)&1ULL) ? ((isprime_bit_table[((n)>>1)/isprime_bit_length] & (1ULL<<(((n)>>1)%isprime_bit_length))) != 0ULL) : ((n)==2))
#endif

/*
  set prime numbers on bits by Eratosthenes's Sieve
*/
inline
void primes_bit_eratosthenes_sieve(isprime_bit_table_element_t *isprime_bit_table, const huge_uint u)
{
  huge_uint i, j;
  const huge_uint n = isprime_bit_table_upper(u);
  const double sqrt_n = sqrt(n);

  memset(isprime_bit_table, ~0, isprime_bit_table_size(n));
  unset_isprime_bit_table(0);
  unset_isprime_bit_table(1);
  for (i = 2; i < sqrt_n; i++)
    for (j = 2; i * j < n; j++)
      unset_isprime_bit_table(i*j);
}
inline
huge_uint *prime_bit_array(isprime_bit_table_element_t *isprime_bit_table, const huge_uint n, size_t *n_primes)
{
  huge_uint i, *a;
  size_t j = 0, c = 0;

  for (i=0; i<n; i++)
    if (isprime_bit(i))
      c++;
  *n_primes = c;
  if (!(a = malloc(sizeof(huge_uint)*c)))
    return NULL;
  for (i=0; i<n; i++)
    if (isprime_bit(i))
      a[j++] = i;
  return a;
}

/*
  add prime numbers on bits by Eratosthenes's Sieve
*/
inline
void primes_bit_continued_eratosthenes_sieve(const huge_uint u_completed, isprime_bit_table_element_t *isprime_bit_table, const huge_uint u)
{
  huge_uint i, j;
  const huge_uint n = isprime_bit_table_upper(u);
  const huge_uint n_completed = isprime_bit_table_upper(u_completed);
  const double sqrt_n = sqrt(n);

  if (!(n_completed < n)) return;
  memset(&isprime_bit_table[isprime_bit_table_length(n_completed)], ~0, isprime_bit_table_size(n-n_completed));
  unset_isprime_bit_table(0);
  unset_isprime_bit_table(1);
  for (i = 2; i < sqrt_n; i++)
    for (j = n_completed/i; i * j < n; j++)
      unset_isprime_bit_table(i*j);
}

/*
  set prime numbers on bits of mmap file
*/
#if !defined(USE_128_BIT_TYPE)
#if defined(DONT_USE_EVEN_OMISSION_IN_BIT_TABLE)
#define isprime_bit_table_path "/var/tmp/isprime_bit_table+even.bin"
#else
#define isprime_bit_table_path "/var/tmp/isprime_bit_table.bin"
#endif
#else
#if defined(DONT_USE_EVEN_OMISSION_IN_BIT_TABLE)
#define isprime_bit_table_path "/var/tmp/isprime_bit_table+even.128bit.bin"
#else
#define isprime_bit_table_path "/var/tmp/isprime_bit_table.128bit.bin"
#endif
#endif

static inline
int isprime_bit_table_write(const isprime_bit_table_element_t *isprime_bit_table, const huge_uint u)
{
  int fd;
  const huge_uint n = isprime_bit_table_upper(u);

  if ((fd = open(isprime_bit_table_path, O_CREAT|O_WRONLY|O_TRUNC, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH)) < 0) return 1;
  if (write(fd, (void *)isprime_bit_table, isprime_bit_table_size(n)) < 0) return 1;
  close(fd);
  return 0;
}
static inline
isprime_bit_table_element_t *isprime_bit_table_mmap_rdonly(huge_uint *n_return, int *fd_return)
{
  int fd;
  isprime_bit_table_element_t *isprime_bit_table = NULL;
  struct stat st;

  if ((fd = open(isprime_bit_table_path, O_RDONLY)) < 0) return isprime_bit_table;
  if (fstat(fd, &st) < 0) { close(fd); return isprime_bit_table; }
#if defined(DONT_USE_EVEN_OMISSION_IN_BIT_TABLE)
  *n_return = st.st_size*8;
#else
  *n_return = st.st_size*16;
#endif
  isprime_bit_table = (isprime_bit_table_element_t *)mmap(NULL, isprime_bit_table_size(*n_return), PROT_READ, MAP_PRIVATE, fd, 0);
  *fd_return = fd;
  return isprime_bit_table;
}
static inline
void isprime_bit_table_munmap_rdonly(const isprime_bit_table_element_t *p, const huge_uint u, const int fd)
{
  const huge_uint n = isprime_bit_table_upper(u);

  munmap((void *)p, isprime_bit_table_size(n));
  close(fd);
}
static inline
isprime_bit_table_element_t *isprime_bit_table_mmap(const huge_uint u, huge_uint *n_return, int *fd_return)
{
  int fd;
  const huge_uint n = isprime_bit_table_upper(u);
  isprime_bit_table_element_t *isprime_bit_table = NULL;
  struct stat st;

  if ((fd = open(isprime_bit_table_path, O_RDWR|O_APPEND)) < 0 ||
      (fd = open(isprime_bit_table_path, O_CREAT|O_RDWR, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH)) < 0) return isprime_bit_table;
  if (fstat(fd, &st) < 0) { close(fd); return isprime_bit_table; }
#if defined(DONT_USE_EVEN_OMISSION_IN_BIT_TABLE)
  *n_return = st.st_size*8;
#else
  *n_return = st.st_size*16;
#endif
  if (*n_return < n) {
    isprime_bit_table_element_t *p;

    if (!(p=malloc(isprime_bit_table_size(n-*n_return)))) { close(fd); return isprime_bit_table; }
    if (lseek(fd, 0, SEEK_END) < 0 || write(fd, (void *)p, isprime_bit_table_size(n-*n_return)) < 0) { free(p); close(fd); return isprime_bit_table; }
    free(p);
    isprime_bit_table = (isprime_bit_table_element_t *)mmap(NULL, isprime_bit_table_size(n), PROT_READ|PROT_WRITE, MAP_SHARED, fd, 0);
  }
  else
    isprime_bit_table = (isprime_bit_table_element_t *)mmap(NULL, isprime_bit_table_size(*n_return), PROT_READ, MAP_PRIVATE, fd, 0);
  *fd_return = fd;
  return isprime_bit_table;
}
static inline
void isprime_bit_table_munmap(const isprime_bit_table_element_t *p, const huge_uint u, const int fd)
{
  const huge_uint n = isprime_bit_table_upper(u);

  munmap((void *)p, isprime_bit_table_size(n));
  close(fd);
}

static inline
huge_uint isprime_bit_table_firstprime(const huge_uint u, const isprime_bit_table_element_t *isprime_bit_table, huge_uint n)
{
  if (isprime_bit(n)) return n;
  while ((++n<u) && !isprime_bit(n)) {}
  return n;
}
static inline
huge_uint isprime_bit_table_nextprime(const huge_uint u, const isprime_bit_table_element_t *isprime_bit_table, huge_uint n)
{
  while ((++n<u) && !isprime_bit(n)) {}
  return n;
}
static inline
huge_uint isprime_bit_table_previousprime(const huge_uint u, const isprime_bit_table_element_t *isprime_bit_table, huge_uint n)
{
  (void)u;
  while ((--n>0) && !isprime_bit(n)) {}
  return n; // 0 means an unnatural number and decrement terminator
}

static inline
huge_uint isprime_bit_table_firstcomposite(const huge_uint u, const isprime_bit_table_element_t *isprime_bit_table, huge_uint n)
{
  if (n != 1 && !isprime_bit(n)) return n;
  while ((++n<u) && isprime_bit(n)) {}
  return n;
}
static inline
huge_uint isprime_bit_table_nextcomposite(const huge_uint u, const isprime_bit_table_element_t *isprime_bit_table, huge_uint n)
{
  while ((++n<u) && isprime_bit(n)) {}
  return n;
}
static inline
huge_uint isprime_bit_table_previouscomposite(const huge_uint u, const isprime_bit_table_element_t *isprime_bit_table, huge_uint n)
{
  (void)u;
  while ((--n>0) && !(n != 1 && !isprime_bit(n))) {}
  return n; // 0 means an unnatural number and decrement terminator
}

static inline
huge_uint *factorize(const huge_uint u, const isprime_bit_table_element_t *isprime_bit_table, huge_uint n, huge_uint *n_return)
{
  huge_uint f = 2;
  huge_uint *p = NULL;
  size_t n_p = 0;

  for (; f <= n; f = isprime_bit_table_nextprime(u, isprime_bit_table, f)) {
    while (n % f == 0) {
      if ((p = p ? realloc(p, sizeof(huge_uint)*(n_p+1)) : malloc(sizeof(huge_uint)*(n_p+1)))) p[n_p++] = f;
      n /= f;
    }
    if (!(f < u)) abort();
  }
  *n_return = n_p;
  return p;
}
inline
huge_uint sum_factorize(const huge_uint u, const isprime_bit_table_element_t *isprime_bit_table, huge_uint n)
{
  huge_uint f = 2, s = 0;

  for (; f <= n; f = isprime_bit_table_nextprime(u, isprime_bit_table, f)) {
    while (n % f == 0) {
      s += f;
      n /= f;
    }
    if (!(f < u)) abort();
  }
  return s;
}

static inline
_Bool is_primitive_pseudoperfect(const huge_uint u, const isprime_bit_table_element_t *isprime_bit_table, huge_uint n)
{
  huge_uint n_d, *d;
  size_t k;

  (void)u;
  if (!is_pseudoperfect_with_proper_divisors(n, &d, &n_d)) { free(d); return 0; }
  for (k=0; k<n_d; ++k) {
    if (isprime_bit(d[k])) continue;
    huge_uint n_pd, *pd = proper_divisors(d[k], &n_pd);
    huge_uint i, c;
    size_t j;
    c = 1U<<n_pd;
    for (i=1; i<c; ++i) {
      huge_uint s = 0;
      for (j=0; j<n_pd; ++j)
	if (i & 1U<<j)
	  s += pd[j];
      if (s == d[k]) break;
    }
    if (i != c) { free(pd); break; }
    free(pd);
  }
  free(d);
  return k == n_d;
}
static inline
_Bool is_non_prime_unusual(const huge_uint u, const isprime_bit_table_element_t *isprime_bit_table, huge_uint n)
{
  huge_uint n_f, *f = factorize(u, isprime_bit_table, n, &n_f);
  huge_uint sqrt_n;
  _Bool r;

  sqrt_n = sqrt(n);
  r = n_f > 1 && f[n_f-1] > sqrt_n;
  free(f);
  return r;
}
static inline
_Bool is_unusual(const huge_uint u, const isprime_bit_table_element_t *isprime_bit_table, const huge_uint n)
{
  huge_uint n_f, *f = factorize(u, isprime_bit_table, n, &n_f);
  huge_uint sqrt_n;
  _Bool r;

  sqrt_n = sqrt(n);
  r = n_f > 0 && f[n_f-1] > sqrt_n;
  free(f);
  return r;
}

static inline
huge_uint *prime_factors(const huge_uint u, const isprime_bit_table_element_t *isprime_bit_table, huge_uint n, huge_uint *n_return)
{
  huge_uint f = 2;
  huge_uint *p = NULL;
  size_t n_p = 0;

  for (; f <= n; f = isprime_bit_table_nextprime(u, isprime_bit_table, f)) {
    if (n % f == 0) {
      if ((p = p ? realloc(p, sizeof(huge_uint)*(n_p+1)) : malloc(sizeof(huge_uint)*(n_p+1)))) p[n_p++] = f;
      n /= f;
    }
    while (n % f == 0)
      n /= f;
    if (!(f < u)) abort();
  }
  *n_return = n_p;
  return p;
}

static inline
_Bool is_round_number(const huge_uint u, const isprime_bit_table_element_t *isprime_bit_table, const huge_uint n)
{
  huge_uint sqrt_n;
  huge_uint n_f, *f = prime_factors(u, isprime_bit_table, n, &n_f);
  size_t i;

  sqrt_n = sqrt(n);
  for (i=0; i!=n_f; ++i)
    if (f[i] > sqrt_n)
      break;
  free(f);
  return i==n_f;
}
static inline
_Bool is_lucas_carmichael(const huge_uint u, const isprime_bit_table_element_t *isprime_bit_table, const huge_uint n)
{
  huge_uint n_f, *f = prime_factors(u, isprime_bit_table, n, &n_f);
  size_t i;

  if (n_f < 3) return 0;
  for (i=0; i!=n_f; ++i)
    if ((n+1) % (f[i]+1) != 0)
      break;
  free(f);
  return i==n_f;
}

inline
huge_uint primorial_number(const huge_uint u, const isprime_bit_table_element_t *isprime_bit_table, huge_uint n)
{
  huge_uint p = 2, f = 1, i = 1;

  for (; i<=n; p = isprime_bit_table_nextprime(u, isprime_bit_table, p), ++i) {
    f *= p;
    if (!(p < u)) abort();
  }
  return f;
}
inline
huge_uint primorial(const huge_uint u, const isprime_bit_table_element_t *isprime_bit_table, huge_uint p)
{
  huge_uint n = 2, f = 1;

  for (; n<=p; n = isprime_bit_table_nextprime(u, isprime_bit_table, n)) {
    f *= n;
    if (!(n < u)) abort();
  }
  return f;
}

inline
huge_uint function_euclid(const huge_uint u, const isprime_bit_table_element_t *isprime_bit_table, const huge_uint i)
{
  return primorial_number(u, isprime_bit_table, i) + 1;
}
inline
huge_uint function_kummer(const huge_uint u, const isprime_bit_table_element_t *isprime_bit_table, const huge_uint i)
{
  return primorial_number(u, isprime_bit_table, i) - 1;
}

static inline
_Bool is_ruth_aaron_pair(const huge_uint u, const isprime_bit_table_element_t *isprime_bit_table, const huge_uint n)
{
  return sum_factorize(u, isprime_bit_table, n) == sum_factorize(u, isprime_bit_table, n+1);
}
static inline
_Bool is_primorial_ruth_aaron_pair(const huge_uint u, const isprime_bit_table_element_t *isprime_bit_table, const huge_uint n)
{
  huge_uint n_f, *f = factorize(u, isprime_bit_table, n, &n_f);
  _Bool r;

  r = sum_factorize(u, isprime_bit_table, n) == sum_factorize(u, isprime_bit_table, n+1) &&
    n*(n+1) == primorial(u, isprime_bit_table, f[n_f-1]);
  free(f);
  return r;
}

inline
_Bool is_powerful(const huge_uint u, const isprime_bit_table_element_t *isprime_bit_table, const huge_uint n)
{
  huge_uint p = 2;

  for (; p<n; p = isprime_bit_table_nextprime(u, isprime_bit_table, p)) {
    if (n % p == 0 && n % (p*p) == 0)
      return !0;
    if (!(p < u)) abort();
  }
  return 0;
}
inline
huge_uint is_perfect_square(const huge_uint u, const isprime_bit_table_element_t *isprime_bit_table, huge_uint n)
{
  huge_uint f = 2, c, c0 = 0;

  for (; f <= n; f = isprime_bit_table_nextprime(u, isprime_bit_table, f)) {
    c = 0;
    while (n % f == 0) {
      ++c;
      n /= f;
    }
    if (c % 2 == 1)
      return 0;
    if (c0 != 0) {
      if (greatest_common_divisor(c0, c) % 2 == 1)
	return 0;
      c0 = greatest_common_divisor(c0, c);
    }
    else
      c0 = c;
    if (!(f < u)) abort();
  }
  return !0;
}
inline
huge_uint is_perfect_power(const huge_uint u, const isprime_bit_table_element_t *isprime_bit_table, huge_uint n)
{
  huge_uint f = 2, c, c0 = 0;

  for (; f <= n; f = isprime_bit_table_nextprime(u, isprime_bit_table, f)) {
    c = 0;
    while (n % f == 0) {
      ++c;
      n /= f;
    }
    if (c == 1)
      return 0;
    if (c0 != 0) {
      if (greatest_common_divisor(c0, c) == 1)
	return 0;
      c0 = greatest_common_divisor(c0, c);
    }
    else
      c0 = c;
    if (!(f < u)) abort();
  }
  return !0;
}
inline
_Bool is_primary_pseudoperfect(const huge_uint u, const isprime_bit_table_element_t *isprime_bit_table, const huge_uint n)
{
  huge_uint n_d, *d = divisors(n, &n_d);
  huge_uint s = 0;
  size_t i;

  (void)u;
  for (i=0; i<n_d; ++i)
    if (isprime_bit(d[i]))
      s += n/(d[i]);
  free(d);
  return n != 1 && s + 1 == n;
}

huge_uint totient_function(const huge_uint u, const isprime_bit_table_element_t *isprime_bit_table, huge_uint n)
{
  huge_uint f = 2, r = n;

  for (; f <= n; f = isprime_bit_table_nextprime(u, isprime_bit_table, f)) {
    if (n % f == 0) {
      r -= r / f;
      n /= f;
    }
    while (n % f == 0)
      n /= f;
    if (!(f < u)) abort();
  }
  return r;
}
huge_uint cototient_function(const huge_uint u, const isprime_bit_table_element_t *isprime_bit_table, huge_uint n)
{
  huge_uint f = 2, r = n;
  const huge_uint c = n;

  for (; f <= n; f = isprime_bit_table_nextprime(u, isprime_bit_table, f)) {
    if (n % f == 0) {
      r -= r / f;
      n /= f;
    }
    while (n % f == 0)
      n /= f;
    if (!(f < u)) abort();
  }
  return c - r;
}
huge_int mebius_function(const huge_uint u, const isprime_bit_table_element_t *isprime_bit_table, huge_uint n)
{
  huge_uint f = 2;
  huge_int r = 1;

  for (; f <= n; f = isprime_bit_table_nextprime(u, isprime_bit_table, f)) {
    if (n % f == 0) {
      r *= -1;
      n /= f;
    }
    if (n % f == 0)
      return 0;
    if (!(f < u)) abort();
  }
  return r;
}

inline
huge_uint function_mersenne(const huge_uint p)
{
  huge_uint m;

  m = 1U<<p;
  return m - 1;
}
inline
huge_uint function_wilson_quotient(const huge_uint p)
{
  huge_uint f;

  f = factorial(p - 1);
  return (f + 1)/p;
}
inline
_Bool is_wilson_prime(const huge_uint n, const huge_uint p)
{
  return n % p == 0;
}

#endif
